@@ -31,9 +31,12 @@ from pytgpt.utils import RawDog
 from WebChatGPT.console import chat as webchatgpt
 from colorama import Fore
 from colorama import init as init_colorama
+from dotenv import load_dotenv
 
 init_colorama(autoreset=True)
 
+load_dotenv()  # loads .env variables
+
 logging.basicConfig(
     format="%(asctime)s - %(levelname)s : %(message)s ",  # [%(module)s,%(lineno)s]", # for debug purposes
     datefmt="%H:%M:%S",
@@ -96,10 +99,18 @@ class this:
             return (False, e)
 
     def g4f_providers_in_dict(
-        url=True, working=True, stream=False, context=False, gpt35=False, gpt4=False
+        url=True,
+        working=True,
+        stream=False,
+        context=False,
+        gpt35=False,
+        gpt4=False,
+        selenium=False,
     ):
         from pytgpt import g4f
+        import g4f.Provider.selenium as selenium_based
 
+        selenium_based_providers: list = dir(selenium_based)
         hunted_providers = []
         required_attrs = (
             "url",
@@ -132,6 +143,13 @@ class this:
                 provider_meta["gpt35_turbo"] = provider.supports_gpt_35_turbo
             if gpt4:
                 provider_meta["gpt4"] = provider.supports_gpt_4
+            if selenium:
+                try:
+                    selenium_based_providers.index(provider_meta["name"])
+                    value = True
+                except ValueError:
+                    value = False
+                provider_meta["non_selenium"] = value
 
             hunted_providers.append(provider_meta)
 
@@ -413,7 +431,9 @@ class Main(cmd.Cmd):
                 )
 
             elif provider == "openai":
-                assert auth, "OpenAI API key is required"
+                assert auth, (
+                    "OpenAI's API-key is required. " "Use the flag `--key` or `-k`"
+                )
                 from pytgpt.openai import main
 
                 self.bot = main.OPENAI(
@@ -553,6 +573,7 @@ class Main(cmd.Cmd):
                     filepath=filepath,
                     update_file=update_file,
                     history_offset=history_offset,
+                    act=awesome_prompt,
                 )
 
             elif provider == "webchatgpt":
@@ -570,6 +591,36 @@ class Main(cmd.Cmd):
                     timeout=timeout,
                     filepath=filepath,
                     update_file=update_file,
+                    intro=intro,
+                    act=awesome_prompt,
+                )
+                intro_response = self.bot.chat(self.bot.conversation.intro, stream=True)
+                if not quiet:
+                    this.stream_output(
+                        intro_response,
+                        title="Intro Response",
+                        is_markdown=True,
+                        style=Style(
+                            color="cyan",
+                        ),
+                    )
+
+            elif provider == "poe":
+                assert auth, (
+                    "Path to poe.com.cookie.json file or 'p-b' cookie-value is required. "
+                    "Use the flag `--key` or `-k`"
+                )
+                from pytgpt.poe import POE, default_model
+
+                self.bot = POE(
+                    cookie=auth,
+                    model=getOr(model, default_model),
+                    proxy=bool(proxies),
+                    timeout=timeout,
+                    filepath=filepath,
+                    update_file=update_file,
+                    intro=intro,
+                    act=awesome_prompt,
                 )
 
             elif provider in pytgpt.gpt4free_providers:
@@ -958,7 +1009,7 @@ class Main(cmd.Cmd):
             busy_bar.stop_spinning()
             is_feedback = self.RawDog.main(ai_response)
             if is_feedback:
-                return self.default(self.bot.chat(is_feedback))
+                return self.default(is_feedback)
             self.__end_time = time.time()
 
         else:
@@ -1189,8 +1240,10 @@ class ChatInteractive:
         default=this.default_provider,
         help="Name of LLM provider.",
         metavar=(
-            f"[{', '.join(pytgpt.tgpt_providers)}] etc, "
-            "run 'pytgpt gpt4free list providers -w' to view more providers"
+            f"[{'|'.join(pytgpt.tgpt_providers)}] etc, "
+            "run 'pytgpt gpt4free list providers -w' to "
+            "view more providers and 'pytgpt gpt4free test -y' "
+            "for advanced g4f providers test"
         ),
     )
     @click.option(
@@ -1485,8 +1538,10 @@ class ChatGenerate:
         default=this.default_provider,
         help="Name of LLM provider.",
         metavar=(
-            f"[{', '.join(pytgpt.tgpt_providers)}] etc, "
-            "run 'pytgpt gpt4free list providers -w' to view more providers"
+            f"[{'|'.join(pytgpt.tgpt_providers)}] etc, "
+            "run 'pytgpt gpt4free list providers -w' to "
+            "view more providers and 'pytgpt gpt4free test -y' "
+            "for advanced g4f providers test"
         ),
     )
     @click.option(
@@ -1639,9 +1694,12 @@ class ChatGenerate:
             else:
                 prompt = prompt + "\n" + stream_text if prompt else stream_text
 
-        elif prompt != prompt.replace(stream_placeholder, ""):
-            # {{stream}} without piped input
-            raise Exception(f"No piped input detected ~ {stream_placeholder}")
+        assert stream_placeholder not in prompt, (
+            "No piped input detected ~ " + stream_placeholder
+        )
+        assert copied_placeholder not in prompt, (
+            "No copied text found ~ " + copied_placeholder
+        )
 
         prompt = Optimizers.code(prompt) if code else prompt
         prompt = Optimizers.shell_command(prompt) if shell else prompt
@@ -1897,9 +1955,15 @@ class Gpt4free:
     @click.option(
         "-4", "--gpt4", is_flag=True, help="Restrict to providers supporting gpt4 model"
     )
+    @click.option(
+        "-se",
+        "--selenium",
+        is_flag=True,
+        help="Restrict to selenium dependent providers",
+    )
     @click.option("-j", "--json", is_flag=True, help="Format output in json")
     @click.help_option("-h", "--help")
-    def show(target, working, url, stream, context, gpt35, gpt4, json):
+    def show(target, working, url, stream, context, gpt35, gpt4, selenium, json):
         """List available models and providers"""
         available_targets = ["models", "providers"]
         assert (
@@ -1919,15 +1983,14 @@ class Gpt4free:
                             context=context,
                             gpt35=gpt35,
                             gpt4=gpt4,
+                            selenium=selenium,
                         ),
                     )
                 )
             )
-            hunted_providers = (
-                hunted_providers[1:]
-                if hunted_providers[0] is None
-                else hunted_providers
-            )
+            while None in hunted_providers:
+                hunted_providers.remove(None)
+
             hunted_providers.sort()
             if json:
                 rich.print_json(data=dict(providers=hunted_providers), indent=4)
@@ -1980,8 +2043,13 @@ class Gpt4free:
                     "llama7b-v2-chat",
                 ],
             )
+            for provider in pytgpt.g4f.Provider.__providers__:
+                if hasattr(provider, "models"):
+                    models[provider.__name__] = provider.models
             if json:
                 for key, value in models.items():
+                    while None in value:
+                        value.remove(None)
                     value.sort()
                     models[key] = value
 
@@ -2043,7 +2111,7 @@ class Gpt4free:
         help="Test n amount of providers at once",
         default=5,
     )
-    @click.option("-q", "--quiet", is_flag=True, help="Suppress all stdout")
+    @click.option("-q", "--quiet", is_flag=True, help="Suppress progress bar")
     @click.option(
         "-j", "--json", is_flag=True, help="Stdout test results in json format"
     )
@@ -2051,13 +2119,33 @@ class Gpt4free:
     @click.option(
         "-b", "--best", is_flag=True, help="Stdout the fastest provider <name only>"
     )
+    @click.option(
+        "-se",
+        "--selenium",
+        help="Test even selenium dependent providers",
+        is_flag=True,
+    )
+    @click.option(
+        "-dl",
+        "--disable-logging",
+        is_flag=True,
+        help="Disable logging",
+    )
     @click.option("-y", "--yes", is_flag=True, help="Okay to all confirmations")
     @click.help_option("-h", "--help")
-    def test(timeout, thread, quiet, json, dry_test, best, yes):
+    def test(
+        timeout, thread, quiet, json, dry_test, best, selenium, disable_logging, yes
+    ):
         """Test and save working providers"""
         from pytgpt.gpt4free import utils
 
-        test = utils.TestProviders(test_at_once=thread, quiet=quiet, timeout=timeout)
+        test = utils.TestProviders(
+            test_at_once=thread,
+            quiet=quiet,
+            timeout=timeout,
+            selenium=selenium,
+            do_log=disable_logging == False,
+        )
         if best:
             click.secho(test.best)
             return
